#!/usr/bin/perl
#
# Copyright 2003 Sergio Rua <srua@debian.org>
# This program is GPL. Please check /usr/share/common-licences/GPL for details.
# 
# Configuration tool for DBM options in openwebmail. Based on dbmtest.pl by
# the openwebmail team.
#

use strict;
use Fcntl qw(:DEFAULT :flock);
use Fcntl;
use FileHandle;

use vars qw(%opentable);
%opentable=();

sub flock_lock {
   my ($filename, $lockflag)=@_;
   my ($dev, $inode, $fh);

   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint...

   if ( (! -e $filename) && $lockflag ne LOCK_UN) {
      sysopen(F, $filename, O_RDWR|O_CREAT, 0600); # create file for lock
      close(F);
   }

   ($dev, $inode)=(stat($filename))[0,1];
   if ($dev eq '' || $inode eq '') {
      return 0;
   }

   if (defined($opentable{"$dev-$inode"}) ) {
      $fh=$opentable{"$dev-$inode"};
   } else { # handle not found, open it!
      $fh=do { local *FH };
      if (sysopen($fh, $filename, O_RDWR) ||	# try RDWR open first
          sysopen($fh, $filename, O_RDONLY) ) {	# then RDONLY for readonly file
         $opentable{"$dev-$inode"}=$fh;
      } else {
         return 0;
      }
   }

   # turn nonblocking lock to  30 secs timeouted lock
   # so owm gets higher chance to success in case other ap locks same file for only few secs
   # turn blocking    lock to 120 secs timeouted lock
   # so openwebmaill won't hang because of file locking
   my $retval;
   eval {
      local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
      if ( $lockflag & LOCK_NB ) {	# nonblocking lock
         alarm 30;
      } else {
         alarm 120;
      }
      $retval=flock($fh, $lockflag & (~LOCK_NB) );
      alarm 0;
   };
   if ($@) {	# eval error, it means timeout
      $retval=0;
   }
   return($retval);
}

sub dbm_test {
   my (%DB, @filelist, @delfiles);
   my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock);

   mkdir ("/tmp/dbmtest.$$", 0755);

   dbmopen(%DB, "/tmp/dbmtest.$$/test", 0600); dbmclose(%DB);

   @delfiles=();
   opendir (TESTDIR, "/tmp/dbmtest.$$");
   while (defined(my $filename = readdir(TESTDIR))) {
      ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint ...
      if ($filename!~/^\./ ) {
         push(@filelist, $filename);
         push(@delfiles, "/tmp/dbmtest.$$/$filename");
      }
   }
   closedir(TESTDIR);
   unlink(@delfiles) if ($#delfiles>=0);

   @filelist=reverse sort(@filelist);
   if ($filelist[0]=~/(\..*)$/) {
      ($dbm_ext, $dbmopen_ext)=($1, '');
   } else {
      ($dbm_ext, $dbmopen_ext)=('.db', '.db');
   }

   flock_lock("/tmp/dbmtest.$$/test$dbm_ext", LOCK_EX);
   eval {
      local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
      alarm 5;	# timeout 5 sec
      dbmopen(%DB, "/tmp/dbmtest.$$/test$dbmopen_ext", 0600); dbmclose(%DB);
      alarm 0;
   };
   if ($@) {	# eval error, it means timeout
      $dbmopen_haslock=1;
   } else {
      $dbmopen_haslock=0;
   }
   flock_lock("/tmp/dbmtest.$$/test$dbm_ext", LOCK_UN);

   @delfiles=();
   opendir (TESTDIR, "/tmp/dbmtest.$$");
   while (defined(my $filename = readdir(TESTDIR))) {
      ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint ...
      push(@delfiles, "/tmp/dbmtest.$$/$filename") if ($filename!~/^\./ );
   }
   closedir(TESTDIR);
   unlink(@delfiles) if ($#delfiles>=0);

   rmdir("/tmp/dbmtest.$$");

   return($dbm_ext, $dbmopen_ext, $dbmopen_haslock);
}


sub fix_dbm_option {
   my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock,$auth_module,$lang,$dictionary)=@_;
   my $dic = 0; # Dictionary could not be defined in the configuration file

   if ($dbmopen_ext eq $dbm_ext) {
      $dbmopen_ext='%dbm_ext%';
   } elsif ($dbmopen_ext eq "") {
      $dbmopen_ext='none';
   }
   if ($dbmopen_haslock) {
      $dbmopen_haslock='yes';
   } else {
      $dbmopen_haslock='no';
   }

   if (-f "/etc/openwebmail/openwebmail.conf") {
	   rename("/etc/openwebmail/openwebmail.conf",
	   	"/etc/openwebmail/openwebmail.conf.$$");
   }

   open S,"</etc/openwebmail/openwebmail.conf.$$" || 
	die "Cannot open openwebmail.conf.$$: $?"; 
   open D,">/etc/openwebmail/openwebmail.conf" || die "Cannot open openwebmail.new: $?"; 
   while(<S>)
   {
	if ($auth_module =~ /auth_pop3.pl/)
	{
  	    $_=~s/^mailspooldir.*/mailspooldir\ \ \ \ \ \ \ \ \ \ \ \ REMOTE/g;
	}
	$_=~s/^auth_module.*/auth_module\ \ \ \ \ \ \ \ \ \ \ \ \ $auth_module/g;
#	$_=~s/^dbm_ext.*/dbm_ext\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ $dbm_ext/g;
#	$_=~s/^dbmopen_ext.*/dbmopen_ext\ \ \ \ \ \ \ \ \ \ \ \ \ \ $dbmopen_ext/g;
#	$_=~s/^dbmopen_haslock.*/dbmopen_haslock\ \ \ \ \ \ \ \ \ \ $dbmopen_haslock/g;
	$_=~s/^default_language.*/default_language\ \ \ \ \ \ \ \ \ \ \ $lang/g;
	$_=~s/^dictionary.*/dictionary\ \ \ \ \ \ \ \ \ \ \ \ \ $dictionary/g;

	print D $_;
   }
   close S;
   close D;

   unlink ("/etc/openwebmail/openwebmail.conf.$$");
   open DBM, ">/etc/openwebmail/dbm.conf" || die "Cannot open dbm.conf: $!\n";
   print DBM "dbm_ext			$dbm_ext\n";
   print DBM "dbmopen_ext		$dbmopen_ext\n";
   print DBM "dbmopen_haslock		$dbmopen_haslock\n";
   close DBM;
}


my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock)=dbm_test();
my ($auth_module)=$ARGV[0];
my ($lang)=$ARGV[1];
my ($dictionary)=$ARGV[2];

fix_dbm_option($dbm_ext, $dbmopen_ext, 
	       $dbmopen_haslock,$auth_module,
	       $lang, $dictionary);

exit(0);
